# SPDX-FileCopyrightText: 2016 Scott Shawcroft for Adafruit Industries
#
# SPDX-License-Identifier: MIT

# pylint: disable=too-few-public-methods

# Ported to Micropython in May 2022 by RetiredWizard for use with modified
# Adafruit_esp32spi library for the purpose of controlling GPIO pins only

"""
`adafruit_bus_device.spi_device` - SPI Bus Device
====================================================
"""

class SPIDevice:
    """
    Represents a single SPI device and manages locking the bus and the device
    address.

    :param ~busio.SPI spi: The SPI bus the device is on
    :param ~digitalio.DigitalInOut chip_select: The chip select pin object that implements the
        DigitalInOut API.
    :param bool cs_active_value: Set to true if your device requires CS to be active high.
        Defaults to false.
    :param int baudrate: The SPI baudrate
    :param int polarity: The SPI polarity
    :param int phase: The SPI phase
    :param int extra_clocks: The minimum number of clock cycles to cycle the bus after CS is high.
        (Used for SD cards.)

    .. note:: This class is **NOT** built into CircuitPython. See
      :ref:`here for install instructions <bus_device_installation>`.

    Example:

    .. code-block:: python

        import busio
        import digitalio
        from board import *
        from adafruit_bus_device.spi_device import SPIDevice

        with busio.SPI(SCK, MOSI, MISO) as spi_bus:
            cs = digitalio.DigitalInOut(D10)
            device = SPIDevice(spi_bus, cs)
            bytes_read = bytearray(4)
            # The object assigned to spi in the with statements below
            # is the original spi_bus object. We are using the busio.SPI
            # operations busio.SPI.readinto() and busio.SPI.write().
            with device as spi:
                spi.readinto(bytes_read)
            # A second transaction
            with device as spi:
                spi.write(bytes_read)
    """

    def __init__(
        self,
        spi: SPI,
        chip_select: Optional[DigitalInOut] = None,
        *,
        cs_active_value: bool = False,
        baudrate: int = 100000,
        polarity: int = 0,
        phase: int = 0,
        extra_clocks: int = 0
    ) -> None:
        self.spi = spi
        self.baudrate = baudrate
        self.polarity = polarity
        self.phase = phase
        self.extra_clocks = extra_clocks
        self.chip_select = chip_select
        self.cs_active_value = cs_active_value
        if self.chip_select:
            self.chip_select.value(True)

    def __enter__(self) -> SPI:
        #while not self.spi.try_lock():
            #pass
        #self.spi.configure(baudrate=self.baudrate, polarity=self.polarity, phase=self.phase)
        if self.chip_select:
            self.chip_select.value(self.cs_active_value)
        return self.spi

    def __exit__(
        self,
        exc_type: Optional[Type[type]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> bool:
        if self.chip_select:
            self.chip_select.value(not self.cs_active_value)
        if self.extra_clocks > 0:
            buf = bytearray(1)
            buf[0] = 0xFF
            clocks = self.extra_clocks // 8
            if self.extra_clocks % 8 != 0:
                clocks += 1
            for _ in range(clocks):
                self.spi.write(buf)
        #self.spi.unlock()
        return False



# PyDOS Board Configuration for 'sparkfun_nrf52840_mini'

import board

Pydos_pins = {
    'sndPin' : (board.A3,"A3 GPIO5"),
    'SCL' : (board.SCL,"SCL GPIO11"),
    'SDA' : (board.SDA,"SDA GPIO8"),
    'SCK' : (board.SCK,"SCK D13 A6 GPIO30"),
    'MOSI' : (board.MOSI,"MOSI D11 A1 GPIO3"),
    'MISO' : (board.MISO,"MISO D12 A7 GPIO31"),
    'CS' : (board.D9,"D9 GPIO10")
}



"""
    Terminal abstraction layer for Keboard FeatherWing by @Arturo182 and @solderparty

    Adds the following keyboard functionality

        Pressing Alt-$ (Speaker) returns an equal sign (=)
        Holding down a key for 2 seconds will either shift the character or return the Alt key
        Pressing Alt-Enter will toggle between Shift and Alt when holding a key for 2 seconds
        Pressing Alt-Space will enable Caps Lock or Alt Lock depending on the Alt-Enter toggle state
        Using a shift key when Caps Lock is on returns the lower case character
        Pressing the Microphone symbol returns a left bracket ([)
        Using the 2 second Shift or Shift Lock on the Microphone Symbol returns a right bracket (])
        Using the 2 second Shift or Shift Lock on the dollar sign ($) returns the percent symbal (%)
        Pressing the F3 key will return the last entered line
        Pressing the F2 and then pressing a key will return the last entered line up to the first instance of that key
        Moving the joystick up/down will scroll through the last 10 entered lines

"""

from bbq10keyboard import BBQ10Keyboard, STATE_PRESS, STATE_RELEASE, STATE_LONG_PRESS
import board
from os import uname
from pydos_hw import Pydos_hw

if board.board_id == 'raspberry_pi_pico':
    import kfw_pico_board as board
elif board.board_id == 'unexpectedmaker_feathers2':
    import kfw_s2_board as board

import adafruit_ili9341
import displayio

class PyDOS_UI:

    SHIFT = 1
    ALT = 2
    #_kbd = None

    def __init__(self):
        # Release any resources currently in use for the displays
        displayio.release_displays()

        _tft_cs = board.D9
        _tft_dc = board.D10
        _display_bus = displayio.FourWire(Pydos_hw.SPI(), command=_tft_dc, chip_select=_tft_cs)
        _display = adafruit_ili9341.ILI9341(_display_bus, width=320,
            height=(235 if int(uname().release.split('.')[0]) >=8 else 240))

        self.kbd = BBQ10Keyboard(Pydos_hw.I2C(),BBQI2CDevice=Pydos_hw.I2CbbqDevice)

        self._contrl_seq = []

        self.shift_Lock = False
        self.shift_Mode = self.SHIFT # Default Cap lock/Long Press mode

        self.lastCmdLine = ""
        self.commandHistory = []

    def serial_bytes_available(self):
        # Does the same function as supervisor.runtime.serial_bytes_available
        try:
            sba = (self.kbd.key_count != 0)
        except:
            print("Trouble on startup, waiting a bit",end="")
            for i in range(100000):
                if i % 10000 == 0:
                    print (".",end="")
            print()
            sba = (self.kbd.key_count != 0)

        return sba


    def process_key(self,key,time_Param):
        # Modified the pressed key value per translation table or
        # held key modifications (shift for 3 seconds, alt for 6 seconds)

        altChars = {'q':'#', 'w':'1', 'e':'2', 'r':'3', 't':'(', 'y':')', 'u':'_', 'i':'-', 'o':'+', 'p':'@',
                    'a':'*', 's':'4', 'd':'5', 'f':'6', 'g':'/', 'h':':', 'j':';', 'k':"'", 'l':'"',
                    'z':'7', 'x':'8', 'c':'9', 'v':'?', 'b':'!', 'n':',', 'm':'.', '$':'=', '~':'0'}

        if time_Param == None and not self.shift_Lock:
            if key == '\x60': # change speaker symbol to equal sign
                retVal = '='
            elif key == '~': # change microphone symbol to left bracket
                retVal = '['
            else:
                retVal = key
        elif self.shift_Mode == self.SHIFT:
            if key == '~':
                retVal = ']'
            elif key == '$':
                retVal = '%'
            else:
                if key != key.upper():
                    retVal = key.upper()
                else:
                    retVal = key.lower()
            print(retVal+'\x08',end="")
        elif self.shift_Mode == self.ALT:
            retVal = altChars.get(key,key)
            print(retVal+'\x08',end="")

        return retVal

    def read_keyboard(self,num):
        # Does the same function as sys.stdin.read(num)
        # Reads num characters from keyboard and returns
        # This is a blocking read, ie the program will wait for the input
        input_text = ""

        while self.kbd.key_count == 0 and len(self._contrl_seq) == 0:
            pass

        while num > 0:
            if len(self._contrl_seq) > 0:
                input_text += self._contrl_seq.pop(0)
                num -= 1
            else:
                k = self.kbd.key
                if k != None:
                    if k[0] == STATE_RELEASE:
                        if k[1] == '\x01':   # Up Arrow - emulate terminal behavior
                            self._contrl_seq = ['\x5b','\x41']
                            input_text += '\x1b'
                            num -= 1
                        elif k[1] == '\x02': # Down Arrow - emulate terminal behavior
                            self._contrl_seq = ['\x5b','\x42']
                            input_text += '\x1b'
                            num -= 1
                        else:
                            input_text += k[1]
                            num -= 1

        return input_text

    def get_screensize(self):
        if int(uname().release.split('.')[0]) >=8:
            return(18,52)
        else:
            return(17,49)

Pydos_ui = PyDOS_UI()

def input(disp_text=None):
    # Does the same function as input(disp_text)
    # displays the "disp_text" prompt and waits for keyboard input
    # This is a blocking read, ie the program will wait for input
    # a carriage return is required for the input to be processed
    input_text = ""

    # There seems to be an initial timing issue so running this
    # extra key_count seems to avoid random startup crashes

    try:
        Pydos_ui.kbd.key_count
    except:
        print("Trouble on startup, waiting a bit",end="")
        for i in range(100000):
            if i % 10000 == 0:
                print (".",end="")
        print()

    if disp_text != None:
        print(disp_text,end='')

    while Pydos_ui.kbd.key_count == 0:
        pass

    k = Pydos_ui.kbd.key
    histPntr = len(Pydos_ui.commandHistory)
    currEditLine = Pydos_ui.lastCmdLine
    loop = True
    time_Param = None
    while loop:
        if k != None:
            if k[0] == STATE_RELEASE:
                if k[1] not in  '\n\x07\x11\x01\x02':
                    if k[1] == '\x08': # Backspace
                        if len(input_text) > 0:
                            input_text = input_text[:-1]
                            print(k[1]+" "+k[1],end="")
                    elif k[1] == '\x09': # Alt-Space toggles the Shift/Alt lock on or off
                        Pydos_ui.shift_Lock = not Pydos_ui.shift_Lock
                    elif k[1] == '\x7c': # Alt-Enter toggles between Shift & Alt for long press
                        if Pydos_ui.shift_Mode == Pydos_ui.SHIFT:
                            Pydos_ui.shift_Mode = Pydos_ui.ALT
                        else:
                            Pydos_ui.shift_Mode = Pydos_ui.SHIFT
                    else: # Anything else, we add to the text field
                        retKey = Pydos_ui.process_key(k[1],time_Param)
                        input_text += retKey
                        time_Param = None
                        print(retKey,end="")
                elif k[1] == '\x01': # Up Arrow
                    if len(Pydos_ui.commandHistory) > 0:
                        print('\x08'*(len(input_text))+" "*(len(input_text))+'\x08'*(len(input_text)),end="")

                        histPntr -= 1
                        if histPntr < 0:
                            histPntr = len(Pydos_ui.commandHistory) - 1
                        print(Pydos_ui.commandHistory[histPntr],end="")
                        input_text = Pydos_ui.commandHistory[histPntr]
                        currEditLine = input_text
                elif k[1] == '\x02': # Down Arrow
                    if len(Pydos_ui.commandHistory) > 0:
                        print('\x08'*(len(input_text))+" "*(len(input_text))+'\x08'*(len(input_text)),end="")

                        histPntr += 1
                        if histPntr >= len(Pydos_ui.commandHistory):
                            histPntr = 0
                        print(Pydos_ui.commandHistory[histPntr],end="")
                        input_text = Pydos_ui.commandHistory[histPntr]
                        currEditLine = input_text
                elif k[1] == '\x07': # F3 pressed

                    print(currEditLine[len(input_text):],end="")
                    input_text += currEditLine[len(input_text):]
                elif k[1] == '\x11': # F2 Pressed
                    searchLoc = currEditLine[len(input_text):].find(Pydos_ui.read_keyboard(1))
                    if searchLoc >= 0:
                        print(currEditLine[len(input_text):len(input_text)+searchLoc],end="")
                        input_text += currEditLine[len(input_text):len(input_text)+searchLoc]
                else: # Enter pressed
                    print()
                    if input_text != "":
                        Pydos_ui.lastCmdLine = input_text
                        Pydos_ui.commandHistory.append(input_text)
                        if len(Pydos_ui.commandHistory) > 10:
                            Pydos_ui.commandHistory.pop(0)
                    loop = False
            elif k[0] == STATE_LONG_PRESS:
                if k[1] not in '\x08\x09\x7c':
                    time_Param = STATE_LONG_PRESS
                    Pydos_ui.process_key(k[1],time_Param)

        if loop:
            k = Pydos_ui.kbd.key

    return input_text



# PyDOS Board Configuration for 'itsybitsy_nrf52840_express'

import board

Pydos_pins = {
    'sndPin' : (board.D12,"GPIO10 D12"),
    'dotStar_Clock' : (board.DOTSTAR_CLOCK,None),
    'dotStar_Data' : (board.DOTSTAR_DATA,None),
    'SCL' : (board.SCL,"D22 SCL"),
    'SDA' : (board.SDA,"D21 SDA"),
    'SCK' : (board.SCK,"D25 SCK"),
    'MOSI' : (board.MOSI,"D24 MO"),
    'MISO' : (board.MISO,"D23 MI"),
    'CS' : (board.D9,"D9")
}



"""Example for Pico (pin 25), Nano Connect (Pin 6). Blinks the built-in LED."""
import time
from machine import Pin
from os import uname
from pydos_hw import Pydos_hw

import _thread
global threadLock
global envVars

if __name__ != "PyDOS":
    envVars = {}

threadLock = _thread.allocate_lock()
envVars['stopthread'] = 'go'

if uname().machine == 'Adafruit Feather RP2040 with RP2040':
    led = Pin(13, Pin.OUT)
elif uname().machine == 'Adafruit ItsyBitsy RP2040 with RP2040':
    led = Pin(11, Pin.OUT)
elif uname().machine == 'Arduino Nano RP2040 Connect with RP2040':
    led = Pin(6, Pin.OUT)
elif Pydos_hw.led:
    led = Pin(Pydos_hw.led, Pin.OUT)
else:
    led = Pin(25, Pin.OUT)

print("blinking started....")

#while not threadLock.locked():
while (not threadLock.locked()) and envVars.get('stopthread') == 'go':
#while True:
    led.value(not led.value())
    time.sleep(0.5)

print()
for i in range(6):
    print(5-i)
    time.sleep(0.5)
print("badblink exiting....")



#import _thread

#global threadLock
#threadLock = _thread.allocate_lock()

if __name__ != "PyDOS":
    passedIn = ""
    global envVars
    envVars = {}

if passedIn == "":
    passedIn = input("Enter python file to launch in thread: ")

cf = open(passedIn)

newF = "import _thread\n"
#newF += "from machine import Timer\nimport sys\n"

newF += "def runAsThread(passedIn):\n"

# Experimenting to see if timer interrupts can shutdown a thread
#     Doesn't seem so....

#newF += "    timer = Timer()\n"
#newF += "    def chkForLockClosure(timer):\n"
#newF += "        global threadLock\n"
#newF += "        if threadLock.locked():\n"
#newF += "            print('DeInit-ing Timer...')\n"
#newF += "            timer.deinit()\n"
#newF += "            print('Raising SystemExit...')\n"
#newF += "            raise SystemExit\n"
#newF += "            print('Executing sys.exit()...')\n"
#newF += "            sys.exit()\n"
#newF += "            print('Executing _thread.exit()...')\n"
#newF += "            _thread.exit()\n"
#newF += "    #timer.init(period=1000, mode=Timer.PERIODIC, callback=chkForLockClosure)\n"

line = " "
while line !="":
    line = cf.readline()
    newF += "    "+line
newF += "\n"
newF += "    if 'stopthread' in envVars.keys():\n"
newF += "        envVars.pop('stopthread')\n"

newF += "\n"
newF += "passedIn = envVars.get('passedIn','')\n"
newF += "envVars['stopthread'] = 'go'\n"
newF += "try:\n"
newF += "    _thread.start_new_thread(runAsThread,(passedIn,))\n"
newF += "except:\n"
newF += '    print("Thread failed, possible core in use")\n'
print(newF)
print('----------- OUTPUT --------------')
print('---------------------------------')
exec(newF)


# PyDOS Board Configuration for 'Teensy 4.1 with MIMXRT1062DVJ6A'

Pydos_pins = {
    'sndPin' : (36,"D36"),
    'led' : (13,None),
    'I2C_NUM' : (None,None),
    'SCL' : (19,"SCL D19 A5"),
    'SDA' : (18,"SDA D18 A4"),
    'SD_SCK' : (44,"CLK D44 Internal"),
    'SD_MOSI' : (45,"CMD D45 Internal"),
    'SD_MISO' : (43,"DAT0 D43 Internal"),
    'SD_CS' : (46,"DAT3 D46 Internal"),
    'SPI_NUM' : (0,None),
    'SCK' : (13,"SCK D13"),
    'MOSI' : (11,"MOSI D11"),
    'MISO' : (12,"MISO D12"),
    'CS' : (10,"D10")
}



import time
import sys
from pydos_hw import Pydos_hw
if sys.implementation.name.upper() == "MICROPYTHON":
    import machine
elif sys.implementation.name.upper() == 'CIRCUITPYTHON':
    from supervisor import ticks_ms
    from pwmio import PWMOut

if __name__ != "PyDOS":
    passedIn = ""

if passedIn=="":
    print("Syntax: sound Frequency,Duration(miliseconds),Volume")
    argv = "1000,0,0"
else:
    argv = passedIn
args = argv.split(",")
if len(args) != 3:
    print("Syntax: sound Frequency,Duration(miliseconds),Volume")
    args = [1000,0,0]

freq=int(args[0])
dur = int(args[1])
vol = int(args[2])

if sys.implementation.name.upper() == "MICROPYTHON":
    pwm=machine.PWM(Pydos_hw.sndPin)
    pwm.freq(freq)
    if 'duty_u16' in dir(pwm):
        pwm.duty_u16(vol)
    else:
        pwm.duty(int((vol/65535)*1023))
    time.sleep(dur/1000)
    if 'duty_u16' in dir(pwm):
        pwm.duty_u16(0)
    else:
        pwm.duty(0)
elif sys.implementation.name.upper() == "CIRCUITPYTHON":
    if not Pydos_hw.sndPin:
        print("Sound Pin not found")
    else:
        Pydos_hw.sndGPIO.deinit() # Workaround for ESP32-S2 GPIO issue
        # Hack for Teensy 4.1 PWM bug
        audioPin = PWMOut(Pydos_hw.sndPin,duty_cycle=0,frequency=freq,variable_frequency=True)
        audioPin.deinit()
        audioPin = PWMOut(Pydos_hw.sndPin,duty_cycle=0,frequency=freq,variable_frequency=True)
        audioPin.deinit()

        audioPin = PWMOut(Pydos_hw.sndPin, duty_cycle=vol, frequency=freq, variable_frequency=True)
        #audioPin.frequency = freq
        #audioPin.duty_cycle = vol
        time.sleep(dur/1000)
        #audioPin.duty_cycle = 0
        audioPin.deinit()
        Pydos_hw.quietSnd() # Workaround for ESP32-S2 GPIO issue



import time
import sys
from pydos_hw import Pydos_hw

if not Pydos_hw.sndPin:
    print("Sound Pin not found")
else:
    if sys.implementation.name.upper() == "MICROPYTHON":
        import machine
        machine.PWM(Pydos_hw.sndPin)
        machine.PWM(Pydos_hw.sndPin)
        piezo=machine.PWM(Pydos_hw.sndPin)
    elif sys.implementation.name.upper() == 'CIRCUITPYTHON':
        from pwmio import PWMOut
        Pydos_hw.sndGPIO.deinit() # Workaround for ESP32-S2 GPIO issue
        piezo = PWMOut(Pydos_hw.sndPin,duty_cycle=0,frequency=440,variable_frequency=True)
        # Hack for Teensy 4.1 PWM bug
        piezo.deinit()
        piezo = PWMOut(Pydos_hw.sndPin,duty_cycle=0,frequency=440,variable_frequency=True)
        piezo.deinit()
        piezo = PWMOut(Pydos_hw.sndPin,duty_cycle=0,frequency=440,variable_frequency=True)

    cmnd = "Y"
    while cmnd.upper() == "Y":
        for f in (200, 300, 400, 500, 600, 700, 800, 900):
            if sys.implementation.name.upper() == "MICROPYTHON":
                piezo.freq(f)
                if 'duty_u16' in dir(piezo):
                    piezo.duty_u16(65535 // 3)
                else:
                    piezo.duty(int(((65535 // 3)/65535)*1023))
                time.sleep(0.25)
                if 'duty_u16' in dir(piezo):
                    piezo.duty_u16(0)
                else:
                    piezo.duty(0)
            elif sys.implementation.name.upper() == "CIRCUITPYTHON":
                piezo.frequency = f
                piezo.duty_cycle = 65535 // 3
                time.sleep(0.25)
                piezo.duty_cycle = 0

            time.sleep(0.05)
        time.sleep(0.5)

        cmnd = input("Again (y/n): ")

    if sys.implementation.name.upper() == "CIRCUITPYTHON":
        piezo.deinit()
        Pydos_hw.quietSnd() # Workaround for ESP32-S2 GPIO issue



# PyDOS Board Configuration for 'SparkFun Thing Plus RP2040 with RP2040'

Pydos_pins = {
    'sndPin' : (17,"D17"),
    'neoPixel' : (8,None),
    'I2C_NUM' : (1,None),
    'SCL' : (7,"SCL D7 D23"),
    'SDA' : (6,"SDA D6"),
    'SD_SPI_NUM' : (1,None),
    'SD_SCK' : (14,"GPIO14 Internal"),
    'SD_MOSI' : (15,"GPIO15 Internal"),
    'SD_MISO' : (12,"GPIO12 Internal"),
    'SD_CS' : (9,"GPIO9 Internal"),
    'SCK' : (2,"SCK D2"),
    'MOSI' : (3,"COPI D3"),
    'MISO' : (4,"CIPO D4"),
    'CS' : (22,"D22")
}




# SPDX-FileCopyrightText: 2021 jfabernathy for Adafruit Industries
# SPDX-License-Identifier: MIT

# adafruit_requests usage with a CircuitPython socket
# this has been tested with Adafruit Metro ESP32-S2 Express

import ssl
import wifi
import socketpool
from os import getenv

import adafruit_requests as requests

# Get wifi details and more from a .env file
if getenv('CIRCUITPY_WIFI_SSID') is None:
    raise Exception("WiFi secrets are kept in .env, please add them there!")

print("Connecting to %s" % getenv('CIRCUITPY_WIFI_SSID'))
wifi.radio.connect(getenv('CIRCUITPY_WIFI_SSID'), getenv('CIRCUITPY_WIFI_PASSWORD'))
print("Connected to %s!" % getenv('CIRCUITPY_WIFI_SSID'))
print("My IP address is", wifi.radio.ipv4_address)

socket = socketpool.SocketPool(wifi.radio)
https = requests.Session(socket, ssl.create_default_context())

#TEXT_URL = "https://money.cnn.com/data/markets"
TEXT_URL = "https://finance.yahoo.com/quote/%5EIXIC"
headers = {"user-agent": "RetiredWizard@circuitpython/8.0.0A1"}

print("Fetching text from %s" % TEXT_URL)
response = https.get(TEXT_URL)
print("-" * 40)
#print("Text Response: ", response.text[0:800])
response_window = []
for _ in range(4):
    response_window.append(next(iter(response.iter_content(chunk_size=256))))
print("Text Response: ", str(b''.join(response_window))[0:800])
print("-" * 40)
#response.close()
print()
#nasdaq = response.text.find('data-ticker-name="Nasdaq"')
#pct = response.text[nasdaq:].find('%')
#pctst = response.text[nasdaq+35:].find('>')
#pctend = response.text[nasdaq+35:].find('<')-1

nasdaq = str(b''.join(response_window)).find('data-symbol="^IXIC" data-field="regularMarketChangePercent"')
while nasdaq == -1:
    for i in range(3):
        response_window[i] = response_window[i+1]
    response_window[3] = next(iter(response.iter_content(chunk_size=256)))

    nasdaq = str(b''.join(response_window)).find('data-symbol="^IXIC" data-field="regularMarketChangePercent"')

for _ in range(4):
    response_window.append(next(iter(response.iter_content(chunk_size=256))))
response.close()

found_window = str(b''.join(response_window))
nasdaq = found_window.find('data-symbol="^IXIC" data-field="regularMarketChangePercent"')

#nasdaq = response.text.find('data-symbol="^IXIC" data-field="regularMarketChangePercent"')
pct = found_window[nasdaq:].find('%)')
pctst = found_window[nasdaq+pct-15:].find('>')
pctend = found_window[nasdaq+pct:].find('<')


#print("Nasdaq: ",response.text[nasdaq+36+pctst:nasdaq+36+pctend])
print("Nasdaq: ",found_window[nasdaq+pct-14+pctst:nasdaq+pct+pctend])

https._free_sockets()



# PyDOS Board Configuration for 'arduino_nano_rp2040_connect'

import board

Pydos_pins = {
    'sndPin' : (board.A3,"GPIO29 A3"),
    'led' : (board.D13,"GPIO6 D13"),
    'SCL' : (board.SCL,"GPIO13 A5"),
    'SDA' : (board.SDA,"GPIO12 A4"),
    'SCK' : (board.SCK,"GPIO6 D13"),
    'MOSI' : (board.MOSI,"GPIO7 D11"),
    'MISO' : (board.MISO,"GPIO4 D12"),
    'SD_CS' : (board.D10,"GPIO5 D10")
}



# PyDOS Board Configuration for 'raspberry_pi_pico'

import board

Pydos_pins = {
    'sndPin' : (board.GP12,"GP12"),
    'SCL'    : (board.GP5,"GP5"),
    'SDA'    : (board.GP4,"GP4"),
    'SCK'    : (board.GP18,"GP18"),
    'MOSI'   : (board.GP19,"GP19"),
    'MISO'   : (board.GP16,"GP16"),
    'CS'     : (board.GP28,"GP28"),
    'SD_CS'  : (board.GP6,"GP6")
}



'''Adapted from the Adafruit_CircuitPython_ESP32SPI
library example esp32spi_simpletest.py:
https://github.com/adafruit/Adafruit_CircuitPython_ESP32SPI/
blob/master/examples/esp32spi_simpletest.py '''

import board
import busio
from digitalio import DigitalInOut
import adafruit_requests as requests
import adafruit_esp32spi.adafruit_esp32spi_socket as socket
from adafruit_esp32spi import adafruit_esp32spi
from os import getenv

# Get wifi details and more from a .env file
if getenv('CIRCUITPY_WIFI_SSID') is None:
    raise Exception("WiFi secrets are kept in .env, please add them there!")

print("Arduino Nano RP2040 Connect webclient test")

TEXT_URL = "http://wifitest.adafruit.com/testwifi/index.html"

#  ESP32 pins
esp32_cs = DigitalInOut(board.CS1)
esp32_ready = DigitalInOut(board.ESP_BUSY)
esp32_reset = DigitalInOut(board.ESP_RESET)

#  uses the secondary SPI connected through the ESP32
spi = busio.SPI(board.SCK1, board.MOSI1, board.MISO1)

esp = adafruit_esp32spi.ESP_SPIcontrol(spi, esp32_cs, esp32_ready, esp32_reset)

requests.set_socket(socket, esp)

if esp.status == adafruit_esp32spi.WL_IDLE_STATUS:
    print("ESP32 found and in idle mode")
print("Firmware vers.", esp.firmware_version)
print("MAC addr:", [hex(i) for i in esp.MAC_address])

#This disconnects from AP
#for ap in esp.scan_networks():
    #print("\t%s\t\tRSSI: %d" % (str(ap['ssid'], 'utf-8'), ap['rssi']))

ntrys = 0
while not esp.is_connected and ntrys < 3:
    if ntrys == 0:
        print("Connecting to AP...")
    ntrys += 1
    try:
        esp.connect_AP(getenv('CIRCUITPY_WIFI_SSID'), getenv('CIRCUITPY_WIFI_PASSWORD'))
    except RuntimeError as e:
        print("could not connect to AP, retrying: ", e)
        continue

if esp.is_connected:
    print("Connected to", str(esp.ssid, "utf-8"), "\tRSSI:", esp.rssi)
    print("My IP address is", esp.pretty_ip(esp.ip_address))

    try:
        print(
            "IP lookup adafruit.com: %s" % esp.pretty_ip(esp.get_host_by_name("adafruit.com"))
        )
    except:
        print("*ERROR* Exception: Failed to request hostname")

    print("Fetching text from", TEXT_URL)
    try:
        r = requests.get(TEXT_URL)
        print("-" * 40)
        print(r.text)
        print("-" * 40)
        r.close()
    except:
        print("*ERROR* Exception: ESP32 not responding")


print("Done!")

esp32_cs.deinit()
esp32_ready.deinit()
esp32_reset.deinit()
spi.deinit()



# PyDOS Board Configuration for 'adafruit_qtpy_esp32s2'

import board

Pydos_pins = {
    'sndPin' : (board.A0,"A0"),
    'neoPixel' : (board.NEOPIXEL,None),
    'SCL' : (board.SCL,"SCL"),
    'SDA' : (board.SDA,"SDA"),
    'SCK' : (board.SCK,"SCK"),
    'MOSI' : (board.MOSI,"MO"),
    'MISO' : (board.MISO,"MI"),
    'CS' : (board.A3,"A3")
}



# The MIT License (MIT)
#
# Copyright (c) 2021 arturo182
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""
`bbq10keyboard`
================================================================================

CircuitPython library for interfacing the BB Q10 Keyboard over I2C.

* Author(s): arturo182

Implementation Notes
--------------------

**Hardware:**

* `BBQ10 Keyboard PMOD <https://www.tindie.com/products/17986/>`_
* `Keyboard FeatherWing <https://www.tindie.com/products/20905/>`_

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the supported boards:
  https://github.com/adafruit/circuitpython/releases

* Adafruit's Bus Device library: https://github.com/adafruit/Adafruit_CircuitPython_BusDevice
"""

from micropython import const
from adafruit_bus_device.i2c_device import I2CDevice
import digitalio
import time

__version__ = "0.2.0-auto.0"
__repo__ = "https://github.com/solderparty/arturo182_CircuitPython_bbq10keyboard.git"


_ADDRESS_KBD = const(0x1F)

_REG_VER = const(0x01)  # fw version
_REG_CFG = const(0x02)  # config
_REG_INT = const(0x03)  # interrupt status
_REG_KEY = const(0x04)  # key status
_REG_BKL = const(0x05)  # backlight
_REG_DEB = const(0x06)  # debounce cfg
_REG_FRQ = const(0x07)  # poll freq cfg
_REG_RST = const(0x08)  # reset
_REG_FIF = const(0x09)  # fifo
_REG_BK2 = const(0x0A)  # backlight 2
_REG_DIR = const(0x0B)  # gpio direction
_REG_PUE = const(0x0C)  # gpio input pull enable
_REG_PUD = const(0x0D)  # gpio input pull direction
_REG_GIO = const(0x0E)  # gpio value
_REG_GIC = const(0x0F)  # gpio interrupt config
_REG_GIN = const(0x10)  # gpio interrupt status

_WRITE_MASK      = const(1 << 7)

CFG_OVERFLOW_ON  = const(1 << 0)
CFG_OVERFLOW_INT = const(1 << 1)
CFG_CAPSLOCK_INT = const(1 << 2)
CFG_NUMLOCK_INT  = const(1 << 3)
CFG_KEY_INT      = const(1 << 4)
CFG_PANIC_INT    = const(1 << 5)
CFG_REPORT_MODS  = const(1 << 6)
CFG_USE_MODS     = const(1 << 7)

INT_OVERFLOW     = const(1 << 0)
INT_CAPSLOCK     = const(1 << 1)
INT_NUMLOCK      = const(1 << 2)
INT_KEY          = const(1 << 3)
INT_PANIC        = const(1 << 4)
INT_GPIO         = const(1 << 5)  # since FW 0.4

KEY_CAPSLOCK     = const(1 << 5)
KEY_NUMLOCK      = const(1 << 6)
KEY_COUNT_MASK   = const(0x1F)

DIR_OUTPUT       = const(0)
DIR_INPUT        = const(1)

PUD_DOWN         = const(0)
PUD_UP           = const(1)

STATE_IDLE       = const(0)
STATE_PRESS      = const(1)
STATE_LONG_PRESS = const(2)
STATE_RELEASE    = const(3)


class DigitalInOut:
    def __init__(self, pin, kbd):
        if pin < 0 or pin > 7:
            raise ValueError('Only pins 0-7 supported right now')

        self._pin = pin
        self._kbd = kbd

    def switch_to_output(self, value=False):
        self.direction = digitalio.Direction.OUTPUT
        self.value = value

    def switch_to_input(self, pull=None):
        self.direction = digitalio.Direction.INPUT
        self.pull = pull

    @property
    def value(self):
        return self._kbd._get_register_bit(_REG_GIO, self._pin)

    @value.setter
    def value(self, value):
        self._kbd._update_register_bit(_REG_GIO, self._pin, value)

    @property
    def direction(self):
        if self._kbd._get_register_bit(_REG_DIR, self._pin) == DIR_INPUT:
            return digitalio.Direction.INPUT

        return digitalio.Direction.OUTPUT

    @direction.setter
    def direction(self, value):
        self._kbd._update_register_bit(_REG_DIR, self._pin, value == digitalio.Direction.INPUT)

    @property
    def pull(self):
        if self.direction == digitalio.Direction.OUTPUT:
            raise AttributeError('Pull not used when direction is output')

        if self._kbd._get_register_bit(_REG_PUE, self._pin):
            if self._kbd._get_register_bit(_REG_PUD, self._pin) == PUD_UP:
                return digitalio.Pull.UP

            return digitalio.Pull.DOWN

        return None

    @pull.setter
    def pull(self, value):
        if self.direction == digitalio.Direction.OUTPUT:
            raise AttributeError('Pull not used when direction is output')

        if value is None:
            self._kbd._update_register_bit(_REG_PUE, self._pin, False)
        else:
            self._kbd._update_register_bit(_REG_PUD, self._pin, value == digitalio.Pull.UP)
            self._kbd._update_register_bit(_REG_PUE, self._pin, True)


class BBQ10Keyboard:
    def __init__(self, i2c, BBQI2CDevice=None):
        self._i2cdelay = 0.01

        if not BBQI2CDevice:
            self._i2c = I2CDevice(i2c, _ADDRESS_KBD)
        else:
            self._i2c = BBQI2CDevice

        self._buffer = bytearray(2)
        self.reset()

    def reset(self):
        with self._i2c as i2c:
            self._buffer[0] = _REG_RST
            time.sleep(self._i2cdelay)
            i2c.write(self._buffer, end=1)

        # need to wait!
        time.sleep(0.05)

    @property
    def version(self):
        ver = self._read_register(_REG_VER)
        return (ver >> 4, ver & 0x0F)

    @property
    def status(self):
        return self._read_register(_REG_KEY)

    @property
    def key_count(self):
        return self.status & KEY_COUNT_MASK

    @property
    def key(self):
        if self.key_count == 0:
            return None

        with self._i2c as i2c:
            self._buffer[0] = _REG_FIF
            time.sleep(self._i2cdelay)
            i2c.write(self._buffer, end=1)
            time.sleep(self._i2cdelay)
            i2c.readinto(self._buffer, end=2)

        return (int(self._buffer[0]), chr(self._buffer[1]))

    @property
    def keys(self):
        keys = []

        for _ in range(self.key_count):
            keys.append(self.key)

        return keys

    @property
    def backlight(self):
        return self._read_register(_REG_BKL) / 255

    @backlight.setter
    def backlight(self, value):
        self._write_register(_REG_BKL, int(255 * value))

    @property
    def backlight2(self):
        if self.version < (0, 4):
            raise NotImplementedError('This function requires FW version 0.4 or newer')

        return self._read_register(_REG_BK2) / 255

    @backlight.setter
    def backlight2(self, value):
        if self.version < (0, 4):
            raise NotImplementedError('This function requires FW version 0.4 or newer')

        self._write_register(_REG_BK2, int(255 * value))

    @property
    def gpio(self):
        return self._read_register(_REG_GIO)

    @gpio.setter
    def gpio(self, value):
        self._write_register(_REG_GIO, value)

    def get_pin(self, pin):
        if pin < 0 or pin > 7:
            raise ValueError('Only pins 0-7 supported right now')

        return DigitalInOut(pin, self)

    def _read_register(self, reg):
        with self._i2c as i2c:
            self._buffer[0] = reg
            time.sleep(self._i2cdelay)
            i2c.write(self._buffer, end=1)
            time.sleep(self._i2cdelay)
            i2c.readinto(self._buffer, end=1)

        return self._buffer[0]

    def _write_register(self, reg, value):
        with self._i2c as i2c:
            self._buffer[0] = reg | _WRITE_MASK
            self._buffer[1] = value
            time.sleep(self._i2cdelay)
            i2c.write(self._buffer, end=2)

    def _update_register_bit(self, reg, bit, value):
        reg_val = self._read_register(reg)
        old_val = reg_val

        if value:
            reg_val |= (1 << bit)
        else:
            reg_val &= ~(1 << bit)

        if reg_val != old_val:
            self._write_register(reg, reg_val)

    def _get_register_bit(self, reg, bit):
        return self._read_register(reg) & (1 << bit) != 0

    keyboard_backlight = backlight
    display_backlight = backlight2




from sys import implementation
if implementation.name.upper() == "MICROPYTHON":
    from os import uname
    envVars["_uname"] = uname().machine
elif implementation.name.upper() == "CIRCUITPYTHON":
    import board
    envVars["_boardID"] = board.board_id



import sys
from pydos_hw import Pydos_hw
from os import uname
try:
    from pydos_ui import input
except:
    pass

if __name__ != "PyDOS":
    passedIn = ""
    envVars = {}

nano_connect = False
pixels = None

if sys.implementation.name.upper() == 'CIRCUITPYTHON':
    import board
    if board.board_id == 'arduino_nano_rp2040_connect':
        import busio
        from digitalio import DigitalInOut
        from adafruit_esp32spi import adafruit_esp32spi

        #  uses the secondary SPI connected through the ESP32
        spi = busio.SPI(board.SCK1, board.MOSI1, board.MISO1)

        #  ESP32 pins
        esp32_cs = DigitalInOut(board.CS1)
        esp32_ready = DigitalInOut(board.ESP_BUSY)
        esp32_reset = DigitalInOut(board.ESP_RESET)

        nano_connect = True

elif sys.implementation.name.upper() == 'MICROPYTHON':
    import machine

    if Pydos_hw.neoPixel_Pow:
        Pydos_hw.neoPixel_Pow.on()

    if uname().machine == 'Arduino Nano RP2040 Connect with RP2040':
        import mp_esp32spi as adafruit_esp32spi

        #  uses the secondary SPI connected through the ESP32
        spi = machine.SPI(1)

        #  ESP32 pins
        esp32_cs = machine.Pin(9,machine.Pin.OUT)
        esp32_ready = machine.Pin(10,machine.Pin.IN)
        esp32_reset = machine.Pin(3,machine.Pin.OUT)

        nano_connect = True

    elif uname().machine == 'TinyPICO with ESP32-PICO-D4':
        from os import umount

        drive = envVars.get('.sd_drive',drive)
        try:
            umount(drive) # Nano Connect uses LED pin for SPI SCK
            print("Unmounting SD card to free up SPI bus")
            envVars.pop('.sd_drive',None)
        except:
            pass

        from dotstar import DotStar
        import tinypico
        spi = machine.SoftSPI(sck=machine.Pin(tinypico.DOTSTAR_CLK),
            mosi=machine.Pin(tinypico.DOTSTAR_DATA), miso=machine.Pin(tinypico.SPI_MISO))
        pixels = DotStar(spi, 1)
        tinypico.set_dotstar_power(True)

if nano_connect:
    #  Nano LED Pins
    LEDG = 25
    LEDB = 26
    LEDR = 27
    OUT = 1   # I/O Direction

    esp = adafruit_esp32spi.ESP_SPIcontrol(spi, esp32_cs, esp32_ready, esp32_reset)

    #esp.set_pin_mode(LEDG,OUT)
    #esp.set_pin_mode(LEDR,OUT)
    #esp.set_pin_mode(LEDB,OUT)

    pixels = True

if not pixels:
    if Pydos_hw.neoPixel or Pydos_hw.dotStar_Clock:
        if ".neopixel" in envVars.keys():
            pixels = envVars['.neopixel']
        elif Pydos_hw.neoPixel:
            import neopixel
            pixels = neopixel.NeoPixel(Pydos_hw.neoPixel, 1)
            # save instance so we can leave neopixel on when we exit
            envVars[".neopixel"] = pixels
        elif Pydos_hw.dotStar_Clock:
            if sys.implementation.name.upper() == 'CIRCUITPYTHON':
                import adafruit_dotstar
                pixels = adafruit_dotstar.DotStar(Pydos_hw.dotStar_Clock, \
                    Pydos_hw.dotStar_Data, 1, auto_write=True)
            elif sys.implementation.name.upper() == 'MICROPYTHON':
                from dotstar import DotStar
                spi = machine.SoftSPI(sck=machine.Pin(Pydos_hw.dotStar_Clock), \
                    mosi=machine.Pin(Pydos_hw.dotStar_Data), miso=machine.Pin(Pydos_hw.MISO))
                pixels = DotStar(spi, 1)
            envVars[".neopixel"] = pixels
    else:
        print("Neopixel/Dotstar not found")

if pixels:
    if passedIn == "":
        ans = input("R,G,B: ")
    else:
        ans = passedIn

    if len(ans.split(",")) == 3:
        r = max(0,min(255,int((ans.split(',')[0] if ans.split(",")[0].isdigit() else 0))))
        g = max(0,min(255,int((ans.split(',')[1] if ans.split(",")[1].isdigit() else 0))))
        b = max(0,min(255,int((ans.split(',')[2] if ans.split(",")[2].isdigit() else 0))))
    else:
        r = 0
        g = 0
        b = 0

    if nano_connect:
        if r+g+b != 0:
            #esp.set_pin_mode(LEDR,OUT)
            #esp.set_pin_mode(LEDG,OUT)
            #esp.set_pin_mode(LEDB,OUT)

            esp.set_analog_write(LEDR,(255-r)/255)
            esp.set_analog_write(LEDG,(255-g)/255)
            esp.set_analog_write(LEDB,(255-b)/255)
        else:
            try:
                esp32_reset.value(0)
            except:
                esp32_reset.value = 0

        try:
            esp32_cs.deinit()
            esp32_ready.deinit()
            esp32_reset.deinit()
            spi.deinit()
        except:
            pass

    elif sys.implementation.name.upper() == 'CIRCUITPYTHON':
        if r+g+b == 0:
            pixels.deinit()
            envVars.pop('.neopixel',None)
        else:
            pixels.fill((r, g, b))

    elif sys.implementation.name.upper() == 'MICROPYTHON':
        if uname().machine == 'TinyPICO with ESP32-PICO-D4':
            pixels.fill((r, g, b))
        else:
            pixels[0] = (r, g, b)
            try:
                pixels.write()
            except:
                pixels.show()
            if r+g+b == 0 and Pydos_hw.neoPixel_Pow:
                Pydos_hw.neoPixel_Pow.off()

        envVars.pop('.neopixel',None)



import os
import time
import sys
try:
    from pydos_ui import input
except:
    pass
if sys.implementation.name.upper() == "MICROPYTHON":
    import uselect

def fileDiff(args):

    def anyKey():

        if sys.implementation.name.upper() == "CIRCUITPYTHON":

            keyIn = input("Press enter to continue . . . .")

        else:

            spoll = uselect.poll()
            spoll.register(sys.stdin,uselect.POLLIN)

            while spoll.poll(0):
                sys.stdin.read(1)

            print("Press any key to continue . . . .",end="")

            keyIn=chr(1)
            while ord(keyIn) < 30 and ord(keyIn) != 10:
                spoll.poll(-1)  # [0][1] != uselect.POLLIN
                keyIn = sys.stdin.read(1)

            spoll.unregister(sys.stdin)
            print("")

        return(keyIn)

    def chkPath(tstPath):
        validPath = True

        simpPath = ""
        if tstPath == []:
            validPath = True
            simpPath = ""
        else:

            savDir = os.getcwd()

            for path in tstPath:
                if path == "":
                    os.chdir("/")

                elif os.getcwd() == "/" and path == "..":
                    validPath = False
                    break

                elif path == ".":
                    continue

                elif path == ".." and len(os.getcwd().split('/')) == 2:
                    os.chdir('/')

                elif path == "..":
                    os.chdir("..")

                elif path in os.listdir() and (os.stat(path)[0] & (2**15) == 0):
                    os.chdir(path)

                else:
                    validPath = False
                    simpPath = ""
                    break

            if validPath:
                simpPath = os.getcwd()
            os.chdir(savDir)

        return((validPath,simpPath))

    def absolutePath(argPath,currDir):

        if argPath[0] == '/':
            fullPath = argPath
        elif currDir == '/':
            fullPath = '/'+argPath
        else:
            fullPath = currDir+'/'+argPath

        if len(fullPath) > 1 and fullPath[-1] == '/':
            fullPath = fullPath[:-1]

        return(fullPath)

    if __name__ == "PyDOS":
        global envVars

        scrLines = int(envVars["_scrHeight"])-1
    else:
        scrLines = 23

    dispdLns = 0

    file1 = args.split(",")[0]
    file2 = args.split(",")[1]

    savDir = os.getcwd()
    file1 = absolutePath(file1,savDir)
    file2 = absolutePath(file2,savDir)

    file1Path = file1.split("/")
    file2Path = file2.split("/")

    file1Dir = file1Path.pop(-1)
    (validPath, tmpDir) = chkPath(file1Path)

    if validPath and file1Dir in os.listdir(tmpDir) and os.stat(tmpDir+("/" if tmpDir[-1] != "/" else "")+file1Dir)[0] & (2**15) != 0:

        file2Dir = file2Path.pop(-1)
        (validPath, tmpDir) = chkPath(file2Path)

        if validPath and file2Dir in os.listdir(tmpDir) and os.stat(tmpDir+("/" if tmpDir[-1] != "/" else "")+file2Dir)[0] & (2**15) != 0:

            f1 = open(file1)
            f2 = open(file2)
            filesSame = True

            line1 = f1.readline()
            fndDiff = False
            sndDiff = False
            lastMatchPos = 0

            while line1 != "":
                f2.seek(lastMatchPos)
                fpos = lastMatchPos
                line2 = f2.readline()

                # find first line2 that matches current line1
                while line2 != "" and line1 != line2:
                    fpos += len(line2)
                    line2 = f2.readline()
                    filesSame = False

                if line1 == line2:

                    # print any mis-matched records from file 2 up until current matched records
                    f2.seek(lastMatchPos)
                    while line2 != "" and lastMatchPos < fpos:
                        line2 = f2.readline()
                        if not sndDiff:
                            dispdLns += 1
                            if dispdLns >= scrLines:
                                anyKey()
                                dispdLns = 0
                            print("***** "+file2+" *****")
                            sndDiff = True
                        dispdLns += 1
                        if dispdLns >= scrLines:
                            anyKey()
                            dispdLns = 0
                        print(line2,end="")
                        lastMatchPos += len(line2)
                        filesSame = False

                    line2 = f2.readline()
                    lastMatchPos = fpos + len(line2)
                    line1 = f1.readline()
                    fndDiff = False
                    sndDiff = False
                else:
                    filesSame = False
                    if not fndDiff:
                        dispdLns += 1
                        if dispdLns >= scrLines:
                            anyKey()
                            dispdLns = 0
                        print("***** "+file1+" *****")
                        fndDiff = True
                    dispdLns += 1
                    if dispdLns >= scrLines:
                        anyKey()
                        dispdLns = 0
                    print(line1,end="")
                    line1 = f1.readline()


            f2.seek(lastMatchPos)
            line2 = f2.readline()
            while line2 != "":
                filesSame = False
                if not sndDiff:
                    dispdLns += 1
                    if dispdLns >= scrLines:
                        anyKey()
                        dispdLns = 0
                    print("***** "+file2+" *****")
                    sndDiff = True
                dispdLns += 1
                if dispdLns >= scrLines:
                    anyKey()
                    dispdLns = 0
                print(line2,end="")
                line2 = f2.readline()

            f1.close()
            f2.close()

            if filesSame:
                print("No differences encountered")

        else:
            print("File not found: "+file2)

    else:
        print("File not found: "+file1)

if __name__ != "PyDOS":
    passedIn = ""

if passedIn == "":
    passedIn = input("Enter filenames (file1,file2):")

if len(passedIn.split(",")) != 2:
    print("Wrong number of arguments, command syntax: diff file1,file2")
else:
    fileDiff(passedIn)



# PyDOS Board Configuration for 'cytron_maker_pi_rp2040'

import board

Pydos_pins = {
    'sndPin' : (board.BUZZER,"board.BUZZER"),
    'neoPixel' : (board.NEOPIXEL,None),
    'SCL' : (board.GP3,"GP3 GROVE#2"),
    'SDA' : (board.GP2,"GP2 GROVE#2"),
    'SD_SCK' : (board.GP14,"GP14 SERVO HEADER"),
    'SD_MOSI' : (board.GP15,"GP15 SERVO HEADER"),
    'SD_MISO' : (board.GP12,"GP12 SERVO HEADER"),
    'SD_CS' : (board.GP5,"GP5 GROVE#3")
}



# Example using PIO to drive a WS2812 LED
# // Copy and paste directly into REPL from TeraTerm
# // Make sure baudrate is set to 115200 bps

import time
from sys import implementation
from pydos_ui import Pydos_ui
from pydos_hw import Pydos_hw
from os import uname

if __name__ != "PyDOS":
    envVars = {}

pixels = None
nano_connect = False
if implementation.name.upper() == 'CIRCUITPYTHON':
    import board
    if board.board_id == 'arduino_nano_rp2040_connect':
        import busio
        from digitalio import DigitalInOut
        from adafruit_esp32spi import adafruit_esp32spi

        #  uses the secondary SPI connected through the ESP32
        spi = busio.SPI(board.SCK1, board.MOSI1, board.MISO1)

        #  ESP32 pins
        esp32_cs = DigitalInOut(board.CS1)
        esp32_ready = DigitalInOut(board.ESP_BUSY)
        esp32_reset = DigitalInOut(board.ESP_RESET)

        nano_connect = True

elif implementation.name.upper() == 'MICROPYTHON':
    import machine

    if Pydos_hw.neoPixel_Pow:
        Pydos_hw.neoPixel_Pow.on()

    if uname().machine == 'Arduino Nano RP2040 Connect with RP2040':
        import mp_esp32spi as adafruit_esp32spi

        #  uses the secondary SPI connected through the ESP32
        spi = machine.SPI(1)

        #  ESP32 pins
        esp32_cs = machine.Pin(9,machine.Pin.OUT)
        esp32_ready = machine.Pin(10,machine.Pin.IN)
        esp32_reset = machine.Pin(3,machine.Pin.OUT)

        nano_connect = True

    elif uname().machine == 'TinyPICO with ESP32-PICO-D4':
        from os import umount
        drive = envVars.get('.sd_drive',drive)
        try:
            umount(drive) # Nano Connect uses LED pin for SPI SCK
            print("Unmounting SD card to free up SPI bus")
            envVars.pop('.sd_drive',None)
        except:
            pass

        from dotstar import DotStar
        import tinypico
        spi = machine.SoftSPI(sck=machine.Pin(tinypico.DOTSTAR_CLK),
            mosi=machine.Pin(tinypico.DOTSTAR_DATA), miso=machine.Pin(tinypico.SPI_MISO))
        pixels = DotStar(spi, 1)
        tinypico.set_dotstar_power(True)

if nano_connect:

    #  Nano LED Pins
    LEDG = 25
    LEDB = 26
    LEDR = 27
    OUT = 1   # I/O Direction

    esp = adafruit_esp32spi.ESP_SPIcontrol(spi, esp32_cs, esp32_ready, esp32_reset)

    pixels = True

if not pixels:
    if Pydos_hw.neoPixel or Pydos_hw.dotStar_Clock:
        if ".neopixel" in envVars.keys():
            pixels = envVars['.neopixel']
            envVars.pop('.neopixel',None)
        elif Pydos_hw.neoPixel:
            import neopixel
            pixels = neopixel.NeoPixel(Pydos_hw.neoPixel, 1)
        elif Pydos_hw.dotStar_Clock:
            if sys.implementation.name.upper() == 'CIRCUITPYTHON':
                import adafruit_dotstar
                pixels = adafruit_dotstar.DotStar(Pydos_hw.dotStar_Clock, \
                    Pydos_hw.dotStar_Data, 1, auto_write=True)
            elif sys.implementation.name.upper() == 'MICROPYTHON':
                from dotstar import DotStar
                spi = machine.SoftSPI(sck=machine.Pin(Pydos_hw.dotStar_Clock), \
                    mosi=machine.Pin(Pydos_hw.dotStar_Data), miso=machine.Pin(Pydos_hw.MISO))
                pixels = DotStar(spi, 1)
    else:
        print("Neopixel not found")

if pixels:
    # attempt to balance colors
    MAXRED = 120
    MAXGREEN = 165
    MAXBLUE = 255

    # Cycle colours.
    icolor = 0
    transition = [-4,1,2,-1,4,1,-2,-1]
    cmnd = ""
    steps = 100
    r = 0
    g = 0
    b = 0

    cmnd = ""
    newSteps = 0
    print("listening... Enter value to alter speed, 'q' to quit")

    while cmnd.upper() != "Q":

        if Pydos_ui.serial_bytes_available():
            cmnd = Pydos_ui.read_keyboard(1)

            if cmnd.upper() == "Q":
                break
            if cmnd == '\n':
                if newSteps == 0:
                    print()
                else:
                    steps = max(1,min(800,newSteps))
                    print(" New STEP value: ",steps)
                    newSteps = 0
            if cmnd.isdigit():
                newSteps = (newSteps * 10) + int(cmnd)
                print(cmnd,end="")
            elif cmnd != None:
                print(cmnd,end="")

        icolor = (icolor + 1) % 8

        for k in range(steps):
            r += ((abs(transition[icolor]) & 1)/transition[icolor]) * (MAXRED / steps)
            b += ((abs(transition[icolor]) & 2)/transition[icolor]) * (MAXBLUE / steps)
            g += ((abs(transition[icolor]) & 4)/transition[icolor]) * (MAXGREEN / steps)
            r = int(max(0,min(MAXRED,r)))
            b = int(max(0,min(MAXBLUE,b)))
            g = int(max(0,min(MAXGREEN,g)))

            if nano_connect:
                esp.set_analog_write(LEDR,(255-r)/255)
                esp.set_analog_write(LEDG,(255-g)/255)
                esp.set_analog_write(LEDB,(255-b)/255)
            elif implementation.name.upper() == 'CIRCUITPYTHON':
                pixels.fill((r,g,b))
            elif implementation.name.upper() == 'MICROPYTHON':
                if uname().machine == 'TinyPICO with ESP32-PICO-D4':
                    pixels.fill((r,g,b))
                else:
                    pixels[0] = (r,g,b)
                    try:
                        pixels.write()
                    except:
                        pixels.show()

            time.sleep(0.5/steps)

        #time.sleep(0.5)

    if nano_connect:
        try:
            esp32_reset.value = 0

            esp32_cs.deinit()
            esp32_ready.deinit()
            esp32_reset.deinit()
            spi.deinit()
        except:
            esp32_reset.value(0)

    if Pydos_hw.neoPixel_Pow:
        try:
            Pydos_hw.neoPixel_Pow.off()
        except:
            pass

    try:
        pixels.deinit()
    except:
        pass



# PyDOS Board Configuration for 'Adafruit Feather RP2040 with RP2040'

Pydos_pins = {
    'sndPin' : (12,"D12 GPIO12"),
    'neoPixel' : (16,None),
    'I2C_NUM' : (1,None),
    'SCL' : (3,"SCL GPIO3"),
    'SDA' : (2,"SDA GPIO2"),
    'SPI_NUM' : (0,None),
    'SCK' : (18,"SCK GPIO18"),
    'MOSI' : (19,"MO GPIO19"),
    'MISO' : (20,"MI GPIO20"),
    'CS' : (7,"D5 GPIO7")
}



# SPDX-FileCopyrightText: 2021 jfabernathy for Adafruit Industries
# SPDX-License-Identifier: MIT

# adafruit_requests usage with a CircuitPython socket
# this has been tested with Adafruit Metro ESP32-S2 Express

import ssl
import wifi
import socketpool
from os import getenv

import adafruit_requests as requests

# Get wifi details and more from a .env file
if getenv('CIRCUITPY_WIFI_SSID') is None:
    raise Exception("WiFi secrets are kept in .env, please add them there!")

print("Connecting to %s" % getenv('CIRCUITPY_WIFI_SSID'))
wifi.radio.connect(getenv('CIRCUITPY_WIFI_SSID'), getenv('CIRCUITPY_WIFI_PASSWORD'))
print("Connected to %s!" % getenv('CIRCUITPY_WIFI_SSID'))
print("My IP address is", wifi.radio.ipv4_address)

socket = socketpool.SocketPool(wifi.radio)
https = requests.Session(socket, ssl.create_default_context())

TEXT_URL = "https://api.weather.gov/gridpoints/BOX/70,76/forecast"
JSON_GET_URL = "https://api.weather.gov/gridpoints/BOX/70,76/forecast"
headers = {"user-agent": "RetiredWizard@circuitpython/7.1.0b0"}

print("Fetching text from %s" % TEXT_URL)
response = https.get(TEXT_URL,headers=headers)
print("-" * 40)
print("Text Response: ", response.text)
print("-" * 40)
response.close()

print("Fetching JSON data from %s" % JSON_GET_URL)
response = https.get(JSON_GET_URL,headers=headers)
print("-" * 40)

print("JSON Response: ", response.json())
print("-" * 40)
print()
print(response.json()['properties']['periods'][0]['detailedForecast'])

https._free_sockets()



# PyDOS Board Configuration for 'adafruit_feather_rp2040'

import board

Pydos_pins = {
    'sndPin' : (board.D12,"D12 GPIO12"),
    'neoPixel' : (board.NEOPIXEL,None),
    'SCL' : (board.SCL,"SCL GPIO3"),
    'SDA' : (board.SDA,"SDA GPIO2"),
    'SCK' : (board.SCK,"SCK GPIO18"),
    'MOSI' : (board.MOSI,"MOSI GPIO19"),
    'MISO' : (board.MISO,"MISO GPIO20"),
    'SD_CS' : (board.D5,"D5 GPIO7")
}



import board
import busio


_SPI = None
_UART = None
_I2C = None


A0 = board.A0
A1 = board.A1
A2 = board.A2
A3 = board.A3
A4 = board.A4
A5 = board.A5
SCK = board.SCK
COPI = board.MOSI
MOSI = board.MOSI
CIPO = board.MISO
MISO = board.MISO
RX = board.RX
TX = board.TX
D14 = board.D4
MISC = board.D4
SCL = board.SCL
SDA = board.SDA
D5 = board.D20
D6 = board.D21
D9 = board.D5
D10 = board.D6
D11 = board.D9
D12 = board.D12
D13 = board.D13

A6 = board.A6
A7 = board.A7
A8 = board.A8
A9 = board.A9
A10 = board.A10
AMB = board.AMB
APA102_MOSI = board.APA102_MOSI
APA102_SCK = board.APA102_SCK
D0 = board.D0
D1 = board.D1
D4 = board.D4
D15 = board.D15
D16 = board.D16
D17 = board.D17
D18 = board.D18
D19 = board.D19
D20 = board.D20
D21 = board.D21
D23 = board.D23
D24 = board.D24
D25 = board.D25
DAC1 = board.DAC1
DAC2 = board.DAC2
IO0 = board.IO0
IO1 = board.IO1
IO3 = board.IO3
IO4 = board.IO4
IO5 = board.IO5
IO6 = board.IO6
IO7 = board.IO7
IO8 = board.IO8
IO9 = board.IO9
IO10 = board.IO10
IO11 = board.IO11
IO12 = board.IO12
IO14 = board.IO14
IO17 = board.IO17
IO18 = board.IO18
IO21 = board.IO21
IO33 = board.IO33
IO35 = board.IO35
IO36 = board.IO36
IO37 = board.IO37
IO38 = board.IO38
IO43 = board.IO43
IO44 = board.IO44
LDO2 = board.LDO2
LED = board.LED
board_id = board.board_id
NEOPIXEL = D11


def SPI():
    global _SPI

    if not _SPI:
        _SPI = board.SPI()

    return _SPI


def UART():
    global _UART

    if not _UART:
        _UART = board.UART()

    return _UART


def I2C():
    global _I2C

    if not _I2C:
        _I2C = board.I2C()

    return _I2C




import sys
from pydos_hw import Pydos_hw
try:
    from pydos_ui import input
except ImportError:
    pass

if sys.implementation.name.upper() == "CIRCUITPYTHON":
    from circuitpython_i2c_lcd import I2cLcd
else:
    import lcd2004

def lcdPrint(passedIn):

    # The PCF8574 has a jumper selectable address: 0x20 - 0x27
    DEFAULT_I2C_ADDR = 0x27

    mess = passedIn
    if mess == "":
        mess = input("Say what?: ")

    i2c = Pydos_hw.I2C()

    if sys.implementation.name.upper() == "CIRCUITPYTHON":
        # circuitpython seems to require locking the i2c bus
        while i2c.try_lock():
            pass

        if DEFAULT_I2C_ADDR not in i2c.scan():
            print("LCD not found at address: ",hex(DEFAULT_I2C_ADDR))
            i2c.unlock()
            return

        # 2 lines, 16 characters per line
        lcd = I2cLcd(i2c, DEFAULT_I2C_ADDR, 2, 16)

        # smiley faces as custom characters
        happy = bytearray([0x00,0x0A,0x00,0x04,0x00,0x11,0x0E,0x00])
        heart = bytearray([0x00,0x00,0x0A,0X15,0X11,0X0A,0X04,0X00])
        grin = bytearray([0x00,0x00,0x0A,0x00,0x1F,0x11,0x0E,0x00])
        lcd.custom_char(0, happy)
        lcd.custom_char(1, heart)
        lcd.custom_char(2, grin)

    else:
        ID=1
        lcd=lcd2004.lcd(DEFAULT_I2C_ADDR,i2c)
        lcd.lcd_backlight(True)
        lcd.lcd_clear()

    if mess == "":
        if sys.implementation.name.upper() == "CIRCUITPYTHON":
            lcd.backlight_off()
        else:
            lcd.lcd_backlight(False)
    else:

        if sys.implementation.name.upper() == "CIRCUITPYTHON":
            mess = mess.replace("<3",chr(1))
            mess = mess.replace(":)",chr(0))
            mess = mess.replace("(:",chr(0))
            mess = mess.replace(":-)",chr(0))
            mess = mess.replace("(-:",chr(0))
            mess = mess.replace(":D",chr(2))

            lcd.move_to(0,0)
            lcd.putstr(mess)
        else:
            lcd.lcd_print(mess,1,0)

    if sys.implementation.name.upper() == "CIRCUITPYTHON":
        i2c.unlock()
        #i2c.deinit()


if __name__ != "PyDOS":
    passedIn = ""

lcdPrint(passedIn)



from sys import implementation
if implementation.name.upper() == "MICROPYTHON":
    from os import umount
elif implementation.name.upper() == "CIRCUITPYTHON":
    from storage import umount

drive = "/sd"

if __name__ != "PyDOS":
    passedIn = ""
    envVars = {}

if passedIn != "":
    drive = passedIn

if drive[0] != '/':
    drive = '/'+drive

if drive == envVars.get('.sd_drive',""):
    envVars.pop('.sd_drive',None)

umount(drive)



# PyDOS Board Configuration for 'swan_r5'

import board

Pydos_pins = {
    'sndPin' : (board.D12,"D12"),
    'SCL' : (board.SCL,"SCL"),
    'SDA' : (board.SDA,"SDA"),
    'SCK' : (board.SCK,"SCK"),
    'MOSI' : (board.MOSI,"MOSI"),
    'MISO' : (board.MISO,"MISO"),
    'CS' : (board.D5,"D5")
}




"""Provides an API for talking to HD44780 compatible character LCDs."""

import time

class LcdApi:
    """Implements the API for talking with HD44780 compatible character LCDs.
    This class only knows what commands to send to the LCD, and not how to get
    them to the LCD.

    It is expected that a derived class will implement the hal_xxx functions.
    """

    # The following constant names were lifted from the avrlib lcd.h
    # header file, however, I changed the definitions from bit numbers
    # to bit masks.
    #
    # HD44780 LCD controller command set

    LCD_CLR = 0x01              # DB0: clear display
    LCD_HOME = 0x02             # DB1: return to home position

    LCD_ENTRY_MODE = 0x04       # DB2: set entry mode
    LCD_ENTRY_INC = 0x02        # --DB1: increment
    LCD_ENTRY_SHIFT = 0x01      # --DB0: shift

    LCD_ON_CTRL = 0x08          # DB3: turn lcd/cursor on
    LCD_ON_DISPLAY = 0x04       # --DB2: turn display on
    LCD_ON_CURSOR = 0x02        # --DB1: turn cursor on
    LCD_ON_BLINK = 0x01         # --DB0: blinking cursor

    LCD_MOVE = 0x10             # DB4: move cursor/display
    LCD_MOVE_DISP = 0x08        # --DB3: move display (0-> move cursor)
    LCD_MOVE_RIGHT = 0x04       # --DB2: move right (0-> left)

    LCD_FUNCTION = 0x20         # DB5: function set
    LCD_FUNCTION_8BIT = 0x10    # --DB4: set 8BIT mode (0->4BIT mode)
    LCD_FUNCTION_2LINES = 0x08  # --DB3: two lines (0->one line)
    LCD_FUNCTION_10DOTS = 0x04  # --DB2: 5x10 font (0->5x7 font)
    LCD_FUNCTION_RESET = 0x30   # See "Initializing by Instruction" section

    LCD_CGRAM = 0x40            # DB6: set CG RAM address
    LCD_DDRAM = 0x80            # DB7: set DD RAM address

    LCD_RS_CMD = 0
    LCD_RS_DATA = 1

    LCD_RW_WRITE = 0
    LCD_RW_READ = 1

    def __init__(self, num_lines, num_columns):
        self.num_lines = num_lines
        if self.num_lines > 4:
            self.num_lines = 4
        self.num_columns = num_columns
        if self.num_columns > 40:
            self.num_columns = 40
        self.cursor_x = 0
        self.cursor_y = 0
        self.implied_newline = False
        self.backlight = True
        self.display_off()
        self.backlight_on()
        self.clear()
        self.hal_write_command(self.LCD_ENTRY_MODE | self.LCD_ENTRY_INC)
        self.hide_cursor()
        self.display_on()

    def clear(self):
        """Clears the LCD display and moves the cursor to the top left
        corner.
        """
        self.hal_write_command(self.LCD_CLR)
        self.hal_write_command(self.LCD_HOME)
        self.cursor_x = 0
        self.cursor_y = 0

    def show_cursor(self):
        """Causes the cursor to be made visible."""
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY |
                               self.LCD_ON_CURSOR)

    def hide_cursor(self):
        """Causes the cursor to be hidden."""
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY)

    def blink_cursor_on(self):
        """Turns on the cursor, and makes it blink."""
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY |
                               self.LCD_ON_CURSOR | self.LCD_ON_BLINK)

    def blink_cursor_off(self):
        """Turns on the cursor, and makes it no blink (i.e. be solid)."""
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY |
                               self.LCD_ON_CURSOR)

    def display_on(self):
        """Turns on (i.e. unblanks) the LCD."""
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY)

    def display_off(self):
        """Turns off (i.e. blanks) the LCD."""
        self.hal_write_command(self.LCD_ON_CTRL)

    def backlight_on(self):
        """Turns the backlight on.

        This isn't really an LCD command, but some modules have backlight
        controls, so this allows the hal to pass through the command.
        """
        self.backlight = True
        self.hal_backlight_on()

    def backlight_off(self):
        """Turns the backlight off.

        This isn't really an LCD command, but some modules have backlight
        controls, so this allows the hal to pass through the command.
        """
        self.backlight = False
        self.hal_backlight_off()

    def move_to(self, cursor_x, cursor_y):
        """Moves the cursor position to the indicated position. The cursor
        position is zero based (i.e. cursor_x == 0 indicates first column).
        """
        self.cursor_x = cursor_x
        self.cursor_y = cursor_y
        addr = cursor_x & 0x3f
        if cursor_y & 1:
            addr += 0x40    # Lines 1 & 3 add 0x40
        if cursor_y & 2:    # Lines 2 & 3 add number of columns
            addr += self.num_columns
        self.hal_write_command(self.LCD_DDRAM | addr)

    def putchar(self, char):
        """Writes the indicated character to the LCD at the current cursor
        position, and advances the cursor by one position.
        """
        if char == '\n':
            if self.implied_newline:
                # self.implied_newline means we advanced due to a wraparound,
                # so if we get a newline right after that we ignore it.
                pass
            else:
                self.cursor_x = self.num_columns
        else:
            self.hal_write_data(ord(char))
            self.cursor_x += 1
        if self.cursor_x >= self.num_columns:
            self.cursor_x = 0
            self.cursor_y += 1
            self.implied_newline = (char != '\n')
        if self.cursor_y >= self.num_lines:
            self.cursor_y = 0
        self.move_to(self.cursor_x, self.cursor_y)

    def putstr(self, string):
        """Write the indicated string to the LCD at the current cursor
        position and advances the cursor position appropriately.
        """
        for char in string:
            self.putchar(char)

    def custom_char(self, location, charmap):
        """Write a character to one of the 8 CGRAM locations, available
        as chr(0) through chr(7).
        """
        location &= 0x7
        self.hal_write_command(self.LCD_CGRAM | (location << 3))
        self.hal_sleep_us(40)
        for i in range(8):
            self.hal_write_data(charmap[i])
            self.hal_sleep_us(40)
        self.move_to(self.cursor_x, self.cursor_y)

    def hal_backlight_on(self):
        """Allows the hal layer to turn the backlight on.

        If desired, a derived HAL class will implement this function.
        """
        pass

    def hal_backlight_off(self):
        """Allows the hal layer to turn the backlight off.

        If desired, a derived HAL class will implement this function.
        """
        pass

    def hal_write_command(self, cmd):
        """Write a command to the LCD.

        It is expected that a derived HAL class will implement this
        function.
        """
        raise NotImplementedError

    def hal_write_data(self, data):
        """Write data to the LCD.

        It is expected that a derived HAL class will implement this
        function.
        """
        raise NotImplementedError

    # This is a default implementation of hal_sleep_us which is suitable
    # for most micropython implementations. For platforms which don't
    # support `time.sleep_us()` they should provide their own implementation
    # of hal_sleep_us in their hal layer and it will be used instead.
    def hal_sleep_us(self, usecs):
        """Sleep for some time (given in microseconds)."""
        time.sleep_us(usecs)  # NOTE this is not part of Standard Python library, specific hal layers will need to override this



# PyDOS Board Configuration for 'sparkfun_thing_plus_rp2040'

import board

Pydos_pins = {
    'sndPin' : (board.D17,"D17"),
    'neoPixel' : (board.NEOPIXEL,None),
    'SCL' : (board.SCL,"SCL D7 D23"),
    'SDA' : (board.SDA,"SDA D6"),
    'SD_SCK' : (board.SD_SCK,"SD_SCK GPIO14 Internal"),
    'SD_MOSI' : (board.SD_MOSI,"SD_MOSI GPIO15 Internal"),
    'SD_MISO' : (board.SD_MISO,"SD_MISO GPIO12 Internal"),
    'SD_CS' : (board.SD_CS,"SD_CS GPIO9 Internal"),
    'SCK' : (board.SCK,"SCK D2"),
    'MOSI' : (board.MOSI,"COPI D3"),
    'MISO' : (board.MISO,"CIPO D4"),
    'CS' : (board.D22,"D22")
}



envline = {}
paramlist = ['CIRCUITPY_WIFI_SSID','CIRCUITPY_WIFI_PASSWORD','CIRCUITPY_WEB_API_PASSWORD']

defaults = True
try:
    envfile = open('/.env')
except:
    defaults = False

if defaults:
    for line in envfile:
        try:
            envline[line.split('=')[0].strip()] = line.split('=')[1].strip()
        except:
            pass
    envfile.close()

ans = ""
while ans.upper() != "Y" and ans.upper() != "A":

    for param in paramlist:
        temp = input(param+": ["+envline.get(param,"")+"] ")
        if temp != "":
            envline[param] = temp

    print("\n/.env file about to be created:\n")
    for param in paramlist:
        print(param+"="+envline.get(param,""))

    print()
    ans = ""
    while ans.upper() != "Y" and ans.upper() != "N" and ans.upper() != "A":
        ans = input("Does this look correct (Y/N/(A)bort)?: ")
        if ans.upper() != "Y" and ans.upper() != "N" and ans.upper() != "A":
            print("Invalid response, please answer Y, N or A")

if ans.upper() != "A":
    envfile = open('/.env','w')
    for param in paramlist:
        envfile.write(param+"="+envline.get(param,"")+"\n")
    envfile.close()



"""
Blink example for QT Py using onboard NeoPixel.
Requires two libraries from the Adafruit CircuitPython Library Bundle.
from circuitpython.org/libraries and copy to your CIRCUITPY/lib folder:
* neopixel.mpy
* adafruit_pypixelbuf.mpy
Save this file as code.py to your CIRCUITPY drive to run it.
"""
import time
import sys
from pydos_ui import Pydos_ui
from pydos_hw import Pydos_hw
from os import uname

if __name__ != "PyDOS":
    envVars = {}

pixels = None
nano_connect = False
if sys.implementation.name.upper() == 'CIRCUITPYTHON':
    import board

    if board.board_id == 'arduino_nano_rp2040_connect':
        import busio
        from digitalio import DigitalInOut
        from adafruit_esp32spi import adafruit_esp32spi

        nano_connect = True

        #  Nano LED Pins
        LEDG = 25
        LEDB = 26
        LEDR = 27

        #  ESP32 pins
        esp32_cs = DigitalInOut(board.CS1)
        esp32_ready = DigitalInOut(board.ESP_BUSY)
        esp32_reset = DigitalInOut(board.ESP_RESET)

        #  uses the secondary SPI connected through the ESP32
        spi = busio.SPI(board.SCK1, board.MOSI1, board.MISO1)

        esp = adafruit_esp32spi.ESP_SPIcontrol(spi, esp32_cs, esp32_ready, esp32_reset)

        pixels = True

elif sys.implementation.name.upper() == 'MICROPYTHON':
    import machine

    if Pydos_hw.neoPixel_Pow:
        Pydos_hw.neoPixel_Pow.on()

    if uname().machine == 'TinyPICO with ESP32-PICO-D4':
        from os import umount
        drive = envVars.get('.sd_drive',drive)
        try:
            umount(drive) # Nano Connect uses LED pin for SPI SCK
            print("Unmounting SD card to free up SPI bus")
            envVars.pop('.sd_drive',None)
        except:
            pass

        from dotstar import DotStar
        import tinypico

        spi = machine.SoftSPI(sck=machine.Pin(tinypico.DOTSTAR_CLK),
            mosi=machine.Pin(tinypico.DOTSTAR_DATA), miso=machine.Pin(tinypico.SPI_MISO))
        pixels = DotStar(spi, 1)
        tinypico.set_dotstar_power(True)
    elif uname().machine == 'Arduino Nano RP2040 Connect with RP2040':
        import mp_esp32spi

        nano_connect = True

        #  Nano LED Pins
        LEDG = 25
        LEDB = 26
        LEDR = 27

        OUT = 1   # I/O Direction

        #  uses the secondary SPI connected through the ESP32
        spi = machine.SPI(1)

        #  ESP32 pins
        esp32_cs = machine.Pin(9,machine.Pin.OUT)
        esp32_ready = machine.Pin(10,machine.Pin.IN)
        esp32_reset = machine.Pin(3,machine.Pin.OUT)

        esp = mp_esp32spi.ESP_SPIcontrol(spi,esp32_cs,esp32_ready,esp32_reset)

        pixels = True

if not pixels:
    if Pydos_hw.neoPixel or Pydos_hw.dotStar_Clock:
        if ".neopixel" in envVars.keys():
            pixels = envVars['.neopixel']
            envVars.pop('.neopixel',None)
        elif Pydos_hw.neoPixel:
            import neopixel
            pixels = neopixel.NeoPixel(Pydos_hw.neoPixel, 1)
        elif Pydos_hw.dotStar_Clock:
            if sys.implementation.name.upper() == 'CIRCUITPYTHON':
                import adafruit_dotstar
                pixels = adafruit_dotstar.DotStar(Pydos_hw.dotStar_Clock, \
                    Pydos_hw.dotStar_Data, 1, auto_write=True)
            elif sys.implementation.name.upper() == 'MICROPYTHON':
                from dotstar import DotStar
                spi = machine.SoftSPI(sck=machine.Pin(Pydos_hw.dotStar_Clock), \
                    mosi=machine.Pin(Pydos_hw.dotStar_Data), miso=machine.Pin(Pydos_hw.MISO))
                pixels = DotStar(spi, 1)
    else:
        print("Neopixel not found")

if pixels:
    icolor = 0
    cmnd = ""

    print("listening..., enter Q to exit")

    while cmnd.upper() != "Q":

        if Pydos_ui.serial_bytes_available():
            cmnd = Pydos_ui.read_keyboard(1)

        icolor = icolor + 1
        icolor = icolor % 3

        if sys.implementation.name.upper() == 'CIRCUITPYTHON':
            if board.board_id == 'arduino_nano_rp2040_connect':
                esp.set_analog_write(LEDR,(0 if icolor == 1 else 1))
                esp.set_analog_write(LEDG,(0 if icolor == 2 else 1))
                esp.set_analog_write(LEDB,(0 if icolor == 0 else 1))
                time.sleep(0.5)
                # Hack to completly turn off RGB LED
                #spi.deinit()
                #spi = busio.SPI(board.SCK1, board.MOSI1, board.MISO1)
                #esp = adafruit_esp32spi.ESP_SPIcontrol(spi, esp32_cs, esp32_ready, esp32_reset)

                # If Hack is used, comment out these 4 lines
                esp.set_analog_write(LEDR,1)
                esp.set_analog_write(LEDG,1)
                esp.set_analog_write(LEDB,1)
                time.sleep(0.5)

            else:
                pixels.fill(((icolor == 1) * 20, (icolor == 2) * 50, (icolor == 0)*150))
                time.sleep(0.5)
                pixels.fill((0, 0, 0))
                time.sleep(0.5)
        elif sys.implementation.name.upper() == 'MICROPYTHON':
            if uname().machine == 'TinyPICO with ESP32-PICO-D4':
                pixels.fill(((icolor == 1) * 20, (icolor == 2) * 50, (icolor == 0)*150))
                time.sleep(0.5)
                pixels.fill((0, 0, 0))
                time.sleep(0.5)
            elif uname().machine == 'Arduino Nano RP2040 Connect with RP2040':
                esp.set_analog_write(LEDR,(0 if icolor == 1 else 1))
                esp.set_analog_write(LEDG,(0 if icolor == 2 else 1))
                esp.set_analog_write(LEDB,(0 if icolor == 0 else 1))
                time.sleep(0.5)

                esp.set_analog_write(LEDR,1)
                esp.set_analog_write(LEDG,1)
                esp.set_analog_write(LEDB,1)
                time.sleep(0.5)
            else:
                pixels[0] = ((icolor == 1) * 20, (icolor == 2) * 50, (icolor == 0)*150)
                try:
                    pixels.write()
                except:
                    pixels.show()
                time.sleep(0.5)
                pixels[0] = (0, 0, 0)
                try:
                    pixels.write()
                except:
                    pixels.show()
                time.sleep(0.5)

    if nano_connect:
        # Hack to completly turn off RGB LED
        #spi.deinit()
        #spi = busio.SPI(board.SCK1, board.MOSI1, board.MISO1)
        #esp = adafruit_esp32spi.ESP_SPIcontrol(spi, esp32_cs, esp32_ready, esp32_reset)
        try:
            esp32_reset.value = 0

            esp32_cs.deinit()
            esp32_ready.deinit()
            esp32_reset.deinit()
            spi.deinit()
        except:
            esp32_reset.value(0)

    if Pydos_hw.neoPixel_Pow:
        try:
            Pydos_hw.neoPixel_Pow.off()
        except:
            pass

    try:
        pixels.deinit()
    except:
        pass
